@@ -9,11 +9,13 @@
 #might try to update
 
 from psychopy import visual, logging, core, event
+visual.useFBO=True#if available (try without for comparison)
+
 import matplotlib
 #matplotlib.use('WXAgg')#change this to control the plotting 'back end'
 import pylab
 
-nIntervals=1000
+nIntervals=500
 win = visual.Window([1280,1024], fullscr=True, allowGUI=False, waitBlanking=True)
 progBar = visual.PatchStim(win, tex=None, mask=None,
     size=[0,0.05],color='red',pos=[0,-0.9],
@@ -37,16 +39,17 @@ for frameN in range(nIntervals+1):
 win.close()
 
 #calculate some values
-intervalsMS = pylab.array(win.frameIntervals[1:])*1000
+intervalsMS = pylab.array(win.frameIntervals)*1000
 m=pylab.mean(intervalsMS)
 sd=pylab.std(intervalsMS)
-distString= "Mean=%.1fms,    s.d.=%.1f,    99%%CI=%.1f-%.1f" %(m,sd,m-3*sd,m+3*sd)
+# se=sd/pylab.sqrt(len(intervalsMS)) # for CI of the mean
+distString= "Mean=%.1fms, s.d.=%.1f, 99%%CI(frame)=%.2f-%.2f" %(m,sd,m-2.58*sd,m+2.58*sd)
 nTotal=len(intervalsMS)
 nDropped=sum(intervalsMS>(1.5*m))
-droppedString = "Dropped/Frames = %i/%i = %.3f%%" %(nDropped,nTotal,nDropped/float(nTotal))
+droppedString = "Dropped/Frames = %i/%i = %.3f%%" %(nDropped,nTotal, 100*nDropped/float(nTotal))
 
 #plot the frameintervals
-pylab.figure(figsize=[20,10])
+pylab.figure(figsize=[12,8])
 pylab.subplot(1,2,1)
 pylab.plot(intervalsMS, '-')
 pylab.ylabel('t (ms)')
