@@ -91,7 +91,7 @@ def _onPygletKey(symbol, modifiers, emulated=False):
     """
 
     global useText
-    keyTime=psychopy.core.getTime() #capture when the key was pressed
+    keyTime=psychopy.clock.getTime() #capture when the key was pressed
     if emulated:
         thisKey = unicode(symbol)
         keySource = 'EmulatedKey'
@@ -114,15 +114,15 @@ def _onPygletMousePress(x,y, button, modifiers):
     global mouseButtons, mouseClick, mouseTimes
     if button == pyglet.window.mouse.LEFT:
         mouseButtons[0]=1
-        mouseTimes[0]= psychopy.core.getTime()-mouseClick[0].timeAtLastReset
+        mouseTimes[0]= psychopy.clock.getTime()-mouseClick[0].getLastResetTime()
         label='Left'
     if button == pyglet.window.mouse.MIDDLE:
         mouseButtons[1]=1
-        mouseTimes[1]= psychopy.core.getTime()-mouseClick[1].timeAtLastReset
+        mouseTimes[1]= psychopy.clock.getTime()-mouseClick[1].getLastResetTime()
         label='Middle'
     if button == pyglet.window.mouse.RIGHT:
         mouseButtons[2]=1
-        mouseTimes[2]= psychopy.core.getTime()-mouseClick[2].timeAtLastReset
+        mouseTimes[2]= psychopy.clock.getTime()-mouseClick[2].getLastResetTime()
         label='Right'
     logging.data("Mouse: %s button down, pos=(%i,%i)" %(label, x,y))
 
@@ -237,8 +237,8 @@ def getKeys(keyList=None, timeStamped=False):
     if timeStamped==False:
         keyNames = [k[0] for k in targets]
         return keyNames
-    elif hasattr(timeStamped, 'timeAtLastReset'):
-        relTuple = [(k[0],k[1]-timeStamped.timeAtLastReset) for k in targets]
+    elif hasattr(timeStamped, 'getLastResetTime'):
+        relTuple = [(k[0],k[1]-timeStamped.getLastResetTime()) for k in targets]
         return relTuple
     elif timeStamped==True:
         return targets
@@ -308,7 +308,17 @@ class Mouse:
         self.visible=visible
         self.lastPos = None
         self.prevPos = None # used for motion detection and timing
-        self.win=win
+        if win:
+            self.win = win
+        else:
+            try:
+                # to avoid circular imports, visualOpenWindows is defined by visual.py
+                # to be the same object as visual.openWindows and is added by visual.py
+                # into event's namespace; it's circular to "import visual" here in event
+                self.win = visualOpenWindows[0]
+                logging.info('Mouse: using default window')
+            except NameError, IndexError:
+                logging.error('Mouse: failed to get a default visual.Window (need to create one first)')
         self.status=None#can be set to STARTED, NOT_STARTED etc for builder
         self.mouseClock=psychopy.core.Clock() # used for movement timing
         self.movedistance=0.0
@@ -410,7 +420,7 @@ class Mouse:
     def mouseMoveTime(self):
         global mouseMove
         if mouseMove:
-                return psychopy.core.getTime()-mouseMove.timeAtLastReset
+                return psychopy.core.getTime()-mouseMove.getLastResetTime()
         else: return 0 # mouseMove clock not started
 
     def getRel(self):
@@ -459,7 +469,7 @@ class Mouse:
 
     def clickReset(self,buttons=[0,1,2]):
         """Reset a 3-item list of core.Clocks use in timing button clicks.
-           The pyglet mouse-button-pressed handler uses their timeAtLastReset when a button is pressed
+           The pyglet mouse-button-pressed handler uses their clock.getLastResetTime() when a button is pressed
            so the user can reset them at stimulus onset or offset to measure RT.
            The default is to reset all, but they can be reset individually as specified in buttons list
         """
@@ -469,9 +479,9 @@ class Mouse:
             mouseTimes[c]=0.0
 
     def getPressed(self, getTime=False):
-        """Returns a 3-item list indicating whether or not buttons 1,2,3 are currently pressed
+        """Returns a 3-item list indicating whether or not buttons 0,1,2 are currently pressed
 
-        If `getTime=True` (False by default( then `getPressed` will return all buttons that
+        If `getTime=True` (False by default) then `getPressed` will return all buttons that
         have been pressed since the last call to `mouse.clickReset` as well as their
         time stamps::
 
@@ -495,6 +505,22 @@ class Mouse:
             if not getTime: return mouseButtons
             else: return mouseButtons, mouseTimes
 
+    def isPressedIn(self, shape, buttons=[0,1,2]):
+        """Returns `True` if the mouse is currently inside the shape and one of the
+        mouse buttons is pressed. The default is that any of the 3 buttons can indicate
+        a click; for only a left-click, specifiy `buttons=[0]`::
+
+            if mouse.isPressedIn(shape):
+            if mouse.isPressedIn(shape, buttons=[0]):  # left-clicks only
+
+        Ideally, `shape` can be anything that has a `.contains()` method, like
+        `ShapeStim` or `Polygon`. Not tested with `ImageStim`.
+        """
+        wanted = numpy.zeros(3, dtype=numpy.int)
+        for c in buttons:
+            wanted[c] = 1
+        pressed = self.getPressed()
+        return any(wanted & pressed) and shape.contains(self)
     def _pix2windowUnits(self, pos):
         if self.win.units=='pix': return pos
         elif self.win.units=='norm': return pos*2.0/self.win.size
