@@ -5,22 +5,27 @@
 # Copyright (C) 2013 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
-# Author: Jeremy R. Gray, March 2012
+# Author: Jeremy R. Gray, March 2012, March 2013
 
 from __future__ import division
 import os, sys, shutil, time
 import threading, urllib2, json
 import tempfile, glob
-from psychopy import core, logging
-from psychopy import sound
-from psychopy.constants import NOT_STARTED, PSYCHOPY_USERAGENT
-# import pyo is done within switchOn/Off to better encapsulate it, because it can be very slow
+import numpy as np
+from scipy.io import wavfile
+from psychopy import core, logging, sound, web
+from psychopy.constants import *
+# import pyo is done within switchOn to better encapsulate it, because it can be very slow
 # idea: don't want to delay up to 3 sec when importing microphone
 # downside: to make this work requires some trickiness with globals
 
 global haveMic
 haveMic = False # goes True in switchOn, if can import pyo
 
+global FLAC_PATH
+FLAC_PATH = None  # set on first call to _getFlacPath(); used for Speech2Text
+
+
 class AudioCapture(object):
     """Capture a sound sample from the default sound input, and save to a file.
 
@@ -60,7 +65,7 @@ class AudioCapture(object):
         Never needed by end-users; only used internally in __init__:
             self.recorder = _Recorder(None) # instantiate, global
         Then in record(), do:
-            self.recorder.run(file, sec)
+            self.recorder.run(filename, sec)
         This sets recording parameters, starts recording.
         To stop a recording that is in progress, do
             self.stop()
@@ -71,34 +76,28 @@ class AudioCapture(object):
         a namespace scoping issue, which using globals seemed to fix; see pyo mailing
         list, 7 April 2012. This draws heavily on Olivier Belanger's solution.
         """
-        def __init__(self, file, sec=0, sampletype=0):
+        def __init__(self):
             self.running = False
-            if file:
-                inputter = Input(chnl=0, mul=1)
-                self.recorder = Record(inputter,
-                               file,
-                               chnls=2,
-                               fileformat=0,
-                               sampletype=sampletype,
-                               buffering=4)
-                self.clean = Clean_objects(sec, self.recorder)
-        def run(self, file, sec, sampletype):
-            self.__init__(file, sec, sampletype)
+        def run(self, filename, sec, sampletype=0, buffering=16, chnl=0, chnls=2):
             self.running = True
-            self.clean.start() # controls recording onset (now) and offset (later)
-            threading.Timer(sec, self._stop).start() # set running flag False
+            inputter = Input(chnl=chnl, mul=1)  # chnl from pyo.pa_get_input_devices()
+            self.recorder = Record(inputter, filename, chnls=chnls, fileformat=0,
+                                sampletype=sampletype, buffering=buffering)
+            Clean_objects(sec, self.recorder).start()  # handles recording offset
+            threading.Timer(sec, self._stop).start()  # set running flag False
         def stop(self):
             self.recorder.stop()
             self._stop()
         def _stop(self):
             self.running = False
 
-    def __init__(self, name='mic', file='', saveDir='', sampletype=0):
+    def __init__(self, name='mic', filename='', saveDir='', sampletype=0,
+                 buffering=16, chnl=0):
         """
         :Parameters:
             name :
                 Stem for the output file, also used in logging.
-            file :
+            filename :
                 optional file name to use; default = 'name-onsetTimeEpoch.wav'
             saveDir :
                 Directory to use for output .wav files.
@@ -107,10 +106,12 @@ class AudioCapture(object):
             sampletype : bit depth
                 pyo recording option: 0=16 bits int, 1=24 bits int; 2=32 bits int
         """
+        if not haveMic:
+            raise MicrophoneError('Need to call microphone.switchOn() before AudioCapture or AdvancedCapture')
         self.name = name
         self.saveDir = saveDir
-        if file:
-            self.wavOutFilename = file
+        if filename:
+            self.wavOutFilename = filename
         else:
             self.wavOutFilename = os.path.join(self.saveDir, name + '.wav')
         if not self.saveDir:
@@ -134,11 +135,9 @@ class AudioCapture(object):
             self.loggingId += ' ' + self.name
 
         # the recorder object needs to persist, or else get bus errors:
-        self.recorder = self._Recorder(None)
-        self.sampletype = sampletype # pass through .run()
+        self.recorder = self._Recorder()
+        self.options = {'sampletype': sampletype, 'buffering': buffering, 'chnl': chnl}
 
-    def __del__(self):
-        pass
     def stop(self):
         """Interrupt a recording that is in progress; close & keep the file.
 
@@ -158,43 +157,48 @@ class AudioCapture(object):
     def reset(self):
         """Restores to fresh state, ready to record again"""
         logging.exp('%s: resetting at %.3f' % (self.loggingId, core.getTime()))
-        self.__del__()
         self.__init__(name=self.name, saveDir=self.saveDir)
-    def record(self, sec, file='', block=True):
+    def record(self, sec, filename='', block=True):
         """Capture sound input for duration <sec>, save to a file.
 
         Return the path/name to the new file. Uses onset time (epoch) as
         a meaningful identifier for filename and log.
         """
+        return self._record(sec, filename=filename, block=block)
+    def _record(self, sec, filename='', block=True):
         while self.recorder.running:
             pass
         self.duration = float(sec)
-        self.onset = core.getTime() # note: report onset time in log, and use in filename
-        logging.data('%s: Record: onset %.3f, capture %.3fs' %
-                     (self.loggingId, self.onset, self.duration) )
-        if not file:
-            onsettime = '-%.3f' % self.onset
+        self.onset = core.getTime()  # for duration estimation, high precision
+        self.fileOnset = core.getAbsTime()  # for log and filename, 1 sec precision
+        ms = "%.3f" % (core.getTime() - int(core.getTime()))
+        logging.data('%s: Record: onset %d, capture %.3fs' %
+                     (self.loggingId, self.fileOnset, self.duration) )
+        if not filename:
+            onsettime = '-%d' % self.fileOnset + ms[1:]
             self.savedFile = onsettime.join(os.path.splitext(self.wavOutFilename))
         else:
-            self.savedFile = os.path.abspath(file).strip('.wav') + '.wav'
+            self.savedFile = os.path.abspath(filename).strip('.wav') + '.wav'
 
         t0 = core.getTime()
-        self.recorder.run(self.savedFile, self.duration, self.sampletype)
-        self.rate = sound.pyoSndServer.getSamplingRate()
+        self.recorder.run(self.savedFile, self.duration, **self.options)
 
+        self.rate = sound.pyoSndServer.getSamplingRate()
         if block:
-            core.wait(self.duration - .0008) # .0008 fudge factor for better reporting
-                # actual timing is done by Clean_objects
+            core.wait(self.duration, 0)
             logging.exp('%s: Record: stop. %.3f, capture %.3fs (est)' %
                      (self.loggingId, core.getTime(), core.getTime() - t0) )
+            while self.recorder.running:
+                core.wait(.001, 0)
         else:
             logging.exp('%s: Record: return immediately, no blocking' %
                      (self.loggingId) )
 
         return self.savedFile
 
-    def playback(self):
-        """Plays the saved .wav file, as just recorded or resampled
+    def playback(self, block=True):
+        """Plays the saved .wav file, as just recorded or resampled. Execution
+        blocks by default, but can return immediately with `block=False`.
         """
         if not self.savedFile or not os.path.isfile(self.savedFile):
             msg = '%s: Playback requested but no saved file' % self.loggingId
@@ -202,8 +206,9 @@ class AudioCapture(object):
             raise ValueError(msg)
 
         # play this file:
-        sound.Sound(self.savedFile).play()
-        core.wait(self.duration) # set during record()
+        sound.Sound(self.savedFile, name=self.name+'.current_recording').play()
+        if block:
+            core.wait(self.duration) # set during record()
 
         logging.exp('%s: Playback: play %.3fs (est) %s' % (self.loggingId, self.duration, self.savedFile))
 
@@ -243,13 +248,13 @@ class AudioCapture(object):
         if not ratio:
             logging.warn('%s: Re-sample by %sx is undefined, skipping' % (self.loggingId, str(ratio)))
         elif self.rate >= newRate:
-            t0 = time.time()
+            t0 = core.getTime()
             downsamp(self.savedFile, newFile, ratio) # default 128-sample anti-aliasing
-            logging.exp('%s: Down-sampled %sx in %.3fs to %s' % (self.loggingId, str(ratio), time.time()-t0, newFile))
+            logging.exp('%s: Down-sampled %sx in %.3fs to %s' % (self.loggingId, str(ratio), core.getTime()-t0, newFile))
         else:
-            t0 = time.time()
+            t0 = core.getTime()
             upsamp(self.savedFile, newFile, ratio) # default 128-sample anti-aliasing
-            logging.exp('%s: Up-sampled %sx in %.3fs to %s' % (self.loggingId, str(ratio), time.time()-t0, newFile))
+            logging.exp('%s: Up-sampled %sx in %.3fs to %s' % (self.loggingId, str(ratio), core.getTime()-t0, newFile))
 
         # clean-up:
         if not keep:
@@ -259,10 +264,209 @@ class AudioCapture(object):
 
         return os.path.abspath(newFile)
 
+class AdvAudioCapture(AudioCapture):
+    """Class extends AudioCapture, plays a marker sound as a "start" indicator.
+
+    Has method for retrieving the marker onset time from the file, to allow
+    calculation of vocal RT (or other sound-based RT).
+
+    See Coder demo > input > latencyFromTone.py
+    """
+    def __init__(self, name='advMic', filename='', saveDir='', sampletype=0,
+                 buffering=16, chnl=0):
+        AudioCapture.__init__(self, name=name, filename=filename, saveDir=saveDir,
+                sampletype=sampletype, buffering=buffering, chnl=chnl)
+        self.setMarker()
+
+    def record(self, sec, filename='', block=False):
+        """Starts recording and plays an onset marker tone just prior
+        to returning. The idea is that the start of the tone in the
+        recording indicates when this method returned, to enable you to sync
+        a known recording onset with other events.
+        """
+        self.filename = self._record(sec, filename=filename, block=block)
+        self.playMarker()
+        return self.filename
+
+    def setFile(self, filename):
+        """Sets the name of the file to work with, e.g., for getting onset time.
+        """
+        self.filename = filename
+    def setMarker(self, tone=19000, secs=0.015, volume=0.03):
+        """Sets the onset marker, where `tone` is either in hz or a custom sound.
+
+        The default tone (19000 Hz) is recommended for auto-detection, as being
+        easier to isolate from speech sounds (and so reliable to detect). The default duration
+        and volume are appropriate for a quiet setting such as a lab testing
+        room. A louder volume, longer duration, or both may give better results
+        when recording loud sounds or in noisy environments, and will be
+        auto-detected just fine (even more easily). If the hardware microphone
+        in use is not physically near the speaker hardware, a louder volume is
+        likely to be required.
+
+        Custom sounds cannot be auto-detected, but are supported anyway for
+        presentation purposes. E.g., a recording of someone saying "go" or
+        "stop" could be passed as the onset marker.
+        """
+        if hasattr(tone, 'play'):
+            self.marker_hz = 0
+            self.marker = tone
+            logging.exp('custom sound set as marker; getMarkerOnset() will not be able to auto-detect onset')
+        else:
+            self.marker_hz = float(tone)
+            sampleRate = sound.pyoSndServer.getSamplingRate()
+            if sampleRate < 2 * self.marker_hz:
+                # NyquistError
+                logging.warning("Recording rate (%i Hz) too slow for %i Hz-based marker detection." % (int(sampleRate), self.marker_hz))
+            logging.exp('frequency of recording onset marker: %.1f' % self.marker_hz)
+            self.marker = sound.Sound(self.marker_hz, secs, volume=volume, name=self.name+'.marker_tone')
+    def playMarker(self):
+        """Plays the current marker sound. This is automatically called at the
+        start of recording, but can be called anytime to insert a marker.
+        """
+        self.marker.play()
+
+    def getMarkerInfo(self):
+        """Returns (hz, duration, volume) of the marker sound.
+        Custom markers always return 0 hz (regardless of the sound).
+        """
+        return self.marker_hz, self.marker.getDuration(), self.marker.getVolume()
+
+    def getMarkerOnset(self, chunk=128, secs=0.5, filename=''):
+        """Return (onset, offset) time of the first marker within the first `secs` of the saved recording.
+
+        Has approx ~1.33ms resolution at 48000Hz, chunk=64. Larger chunks can
+        speed up processing times, at a sacrifice of some resolution, e.g., to
+        pre-process long recordings with multiple markers.
+
+        If given a filename, it will first set that file as the one to work with,
+        and then try to detect the onset marker.
+        """
+        def thresh2SD(data, mult=2, thr=None):
+            """Return index of first value in abs(data) exceeding 2 * std(data),
+            or length of the data + 1 if nothing > threshold
+
+            Return threshold so can re-use the same threshold later
+            """
+            # this algorithm could use improvement
+            data = abs(data)
+            if not thr:
+                thr = mult * np.std(data)
+            val = data[(data > thr)]
+            if not len(val):
+                return len(data)+1, thr
+            first = val[0]
+            for i, v in enumerate(data):
+                if v == first:
+                    return i, thr
+
+        while self.recorder.running:
+            core.wait(0.10, 0)
+        # read from self.filename:
+        if filename:
+            self.setFile(filename)
+        data, sampleRate = readWavFile(self.filename)
+        if self.marker_hz == 0:
+            raise ValueError("Custom marker sounds cannot be auto-detected.")
+        if sampleRate < 2 * self.marker_hz:
+            # NyquistError
+            raise ValueError("Recording rate (%i Hz) too slow for %i Hz-based marker detection." % (int(sampleRate), self.marker_hz))
+
+        # extract onset:
+        chunk = max(16, chunk)  # trades-off against size of bandpass filter
+          # precision in time-domain (= smaller chunks) requires wider freq
+        bandSize = 150 * 2 ** (8 - int(np.log2(chunk)))  # {16: 2400, 32: 1200, 64: 600, 128: 300}
+        dataToUse = data[:int(sampleRate * secs)]  # only look at first secs
+        lo = max(0, self.marker_hz - bandSize)  # for bandpass filter
+        hi = self.marker_hz + bandSize
+        dftProfile = getDftBins(dataToUse, sampleRate, lo, hi, chunk)
+        onsetChunks, thr = thresh2SD(dftProfile)  # leading edge of startMarker in chunks
+        onsetSecs = onsetChunks * chunk / sampleRate  # in secs
+
+        # extract offset:
+        start = onsetChunks - 4
+        stop = int(onsetChunks+self.marker.getDuration()*sampleRate/chunk) + 4
+        backwards = dftProfile[max(start,0):min(stop, len(dftProfile))]
+        offChunks, _ = thresh2SD(backwards[::-1], thr=thr)
+        offSecs = (start + len(backwards) - offChunks) * chunk / sampleRate  # in secs
+
+        return onsetSecs, offSecs
+
+def readWavFile(filename):
+    """Return (data, sampleRate) as read from a wav file
+    """
+    try:
+        sampleRate, data = wavfile.read(filename)
+    except:
+        if os.path.exists(filename) and os.path.isfile(filename):
+            core.wait(0.01, 0)
+        try:
+            sampleRate, data = wavfile.read(filename)
+        except:
+            raise SoundFileError('Failed to open wav sound file "%s"' % filename)
+    if len(data.shape) == 2 and data.shape[1] == 2:
+        data = data.transpose()
+        data = data[0]  # left channel only? depends on how the file was made
+    return data, sampleRate
+def getDftBins(data=[], sampleRate=None, low=100, high=8000, chunk=64):
+    """Get DFT (discrete Fourier transform) of `data`, doing so in time-domain
+    bins of `chunk` samples.
+
+    e.g., for getting FFT magnitudes in a ms-by-ms manner.
+
+    If given a sampleRate, the data are bandpass filtered (low, high).
+    """
+    # good to reshape & vectorize data rather than use a python loop
+    bins = []
+    i = chunk
+    if sampleRate:
+        _, freq = getDft(data[:chunk], sampleRate)  # just to get freq vector
+        band = (freq > low) & (freq < high)  # band (frequency range)
+    while i <= len(data):
+        magn = getDft(data[i-chunk:i])
+        if sampleRate:
+            bins.append(np.std(magn[band]))  # filtered by frequency
+        else:
+            bins.append(np.std(magn))  # unfiltered
+        i += chunk
+    return np.array(bins)
+def getDft(data, sampleRate=None, wantPhase=False):
+    """Compute and return magnitudes of numpy.fft.fft() of the data.
+
+    If given a sample rate (samples/sec), will return (magn, freq).
+    If wantPhase is True, phase in radians is also returned (magn, freq, phase).
+    data should have power-of-2 samples, or will be truncated.
+    """
+    # www.vibrationdata.com/Shock_and_Vibration_Signal_Analysis.pdf and .../python/fft.py
+    # truncate to power-of-2 slice; zero-padding to round up is ok too
+    samples = 2 ** int(np.log2(len(data)))
+    samplesHalf = samples // 2
+    dataSlice = data[:samples]
+
+    # get magn & phase from the DFT:
+    dft = np.fft.fft(dataSlice)
+    dftHalf = dft[:samplesHalf] / samples
+    magn = abs(dftHalf) * 2
+    magn[0] /= 2.
+    if wantPhase:
+        phase = np.arctan2(dftHalf.real, dftHalf.imag)  # in radians
+    if sampleRate:
+        deltaf = sampleRate / samplesHalf / 2.
+        freq = np.linspace(0, samplesHalf * deltaf, samplesHalf, endpoint=False)
+        if wantPhase:
+            return magn, freq, phase
+        return magn, freq
+    else:
+        if wantPhase:
+            return magn, phase
+        return magn
+
 class SoundFormatNotSupported(StandardError):
     """Class to report an unsupported sound format"""
 class SoundFileError(StandardError):
     """Class to report sound file failed to load"""
+class MicrophoneError(StandardError):
+    """Class to report a microphone error"""
 
 class _GSQueryThread(threading.Thread):
     """Internal thread class to send a sound file to google, stash the response.
@@ -271,7 +475,7 @@ class _GSQueryThread(threading.Thread):
         threading.Thread.__init__(self, None, 'GoogleSpeechQuery', None)
 
         # request is a previously established urllib2.request() obj, namely:
-        # request = urllib2.Request(url, audio, header) at end of GoogleSpeech.__init__
+        # request = urllib2.Request(url, audio, header) at end of Speech2Text.__init__
         self.request = request
 
         # set vars and flags:
@@ -378,7 +582,7 @@ class Speech2Text(object):
             while resp.running:
                 print '.', # displays dots while waiting
                 sys.stdout.flush()
-                time.sleep(0.1)
+                core.wait(0.1)
             print resp.words
 
         d) Options: Set-up with a different language for the same speech clip; you'll get a different response (possibly having UTF-8 characters)::
@@ -388,30 +592,21 @@ class Speech2Text(object):
 
         :Example:
 
-            See Coder demos / input / say_rgb.py -- be sure to read the text at the top of the file.
-            The demo works better when run from the command-line than from the Coder.
-
-        :Error handling:
-
-            If there is an error during http connection, it is handled as a lack of
-            response. The connection was probably lost if you get: `WARNING <urlopen error [Errno 8] nodename nor servname provided, or not known>`
+            See Coder demos / input / speech_recognition.py
 
         :Known limitations:
 
-            a) Availability is subject to the whims of google. Any changes google
+            Availability is subject to the whims of google. Any changes google
             makes along the way could either cause complete failure (disruptive),
             or could cause slightly different results to be obtained (without it being
             readily obvious that something had changed). For this reason,
             its probably a good idea to re-run speech samples through `Speech2Text` at the end of
             a study; see `BatchSpeech2Text()`.
 
-            b) Only tested with: Win XP-sp2,
-            Mac 10.6.8 (python 2.6, 2.7).
-
         :Author: Jeremy R. Gray, with thanks to Lefteris Zafiris for his help
             and excellent command-line perl script at https://github.com/zaf/asterisk-speech-recog (GPLv2)
     """
-    def __init__(self, file,
+    def __init__(self, filename,
                  lang='en-US',
                  timeout=10,
                  samplingrate=16000,
@@ -421,7 +616,7 @@ class Speech2Text(object):
         """
             :Parameters:
 
-                `file` : <required>
+                `filename` : <required>
                     name of the speech file (.flac, .wav, or .spx) to process. wav files will be
                     converted to flac, and for this to work you need to have flac (as an
                     executable). spx format is speex-with-headerbyte (for google).
@@ -433,6 +628,7 @@ class Speech2Text(object):
                     the sampling rate of the speech clip in Hz, either 16000 or 8000. You can
                     record at a higher rate, and then down-sample to 16000 for speech
                     recognition. `file` is the down-sampled file, not the original.
+                    the sampling rate is auto-detected for .wav files.
                 `flac_exe` :
                     **Windows only**: path to binary for converting wav to flac;
                     must be a string with **two back-slashes where you want one** to appear
@@ -442,54 +638,55 @@ class Speech2Text(object):
                     profanity filter level; default 2 (e.g., f***)
                 `quiet` :
                     no reporting intermediate details; default `True` (non-verbose)
-
         """
         # set up some key parameters:
         results = 5 # how many words wanted
         self.timeout = timeout
         useragent = PSYCHOPY_USERAGENT
         host = "www.google.com/speech-api/v1/recognize"
-        if sys.platform == 'win32':
-            FLAC_PATH = flac_exe
-        else:
-            # best not to do every time
-            FLAC_PATH, _ = core.shellCall(['/usr/bin/which', 'flac'], stderr=True)
+        flac_path = _getFlacPath(flac_exe)
 
         # determine file type, convert wav to flac if needed:
-        ext = os.path.splitext(file)[1]
-        if not os.path.isfile(file):
+        ext = os.path.splitext(filename)[1]
+        if not os.path.isfile(filename):
             raise IOError("Cannot find file: %s" % file)
         if ext not in ['.flac', '.spx', '.wav']:
             raise SoundFormatNotSupported("Unsupported filetype: %s\n" % ext)
-        self.file = file
+        if ext == '.wav':
+            __, samplingrate = readWavFile(filename)
+        if samplingrate not in [16000, 8000]:
+            raise SoundFormatNotSupported('Speech2Text sample rate must be 16000 or 8000 Hz')
+        self.filename = filename
         if ext == ".flac":
             filetype = "x-flac"
         elif ext == ".spx":
             filetype = "x-speex-with-header-byte"
         elif ext == ".wav": # convert to .flac
-            if not os.path.isfile(FLAC_PATH):
-                sys.exit("failed to find flac")
+            if not os.path.isfile(flac_path):
+                msg = "failed to find flac binary, tried '%s'" % flac_path
+                logging.error(msg)
+                raise MicrophoneError(msg)
             filetype = "x-flac"
-            tmp = 'tmp_guess%.6f' % time.time()+'.flac'
-            flac_cmd = [FLAC_PATH, "-8", "-f", "--totally-silent", "-o", tmp, file]
-            _, se = core.shellCall(flac_cmd, stderr=True)
+            tmp = tempfile.NamedTemporaryFile()
+            flac_cmd = [flac_path, "-8", "-f", "--totally-silent", "-o", tmp.name, filename]
+            __, se = core.shellCall(flac_cmd, stderr=True)
             if se: logging.warn(se)
-            while not os.path.isfile(tmp): # just try again
+            if not os.path.isfile(tmp.name): # just try again
                 # ~2% incidence when recording for 1s, 650+ trials
-                # never got two in a row; time.sleep() does not help
+                # never got two in a row; core.wait() does not help
                 logging.warn('Failed to convert to tmp.flac; trying again')
-                _, se = core.shellCall(flac_cmd, stderr=True)
+                __, se = core.shellCall(flac_cmd, stderr=True)
                 if se: logging.warn(se)
-            file = tmp # note to self: ugly & confusing to switch up like this
-        logging.info("Loading: %s as %s, audio/%s" % (self.file, lang, filetype))
+            filename = tmp.name
+        logging.info("Loading: %s as %s, audio/%s" % (self.filename, lang, filetype))
         try:
-            c = 0 # occasional error; time.sleep(.1) is not always enough; better slow than fail
-            while not os.path.isfile(file) and c < 10:
-                time.sleep(.1)
+            c = 0 # occasional error; core.wait(.1) is not always enough; better slow than fail
+            while not os.path.isfile(filename) and c < 10:
+                core.wait(.1, hogCPUperiod=0)
                 c += 1
-            audio = open(file, 'r+b').read()
+            audio = open(filename, 'r+b').read()
         except:
-            msg = "Can't read file %s from %s.\n" % (file, self.file)
+            msg = "Can't read file %s from %s.\n" % (filename, self.filename)
             logging.error(msg)
             raise SoundFileError(msg)
         finally:
@@ -506,14 +703,13 @@ class Speech2Text(object):
               'maxresults=%d' % results
         header = {'Content-Type' : 'audio/%s; rate=%d' % (filetype, samplingrate),
                   'User-Agent': useragent}
+        web.requireInternetAccess()  # needed to access google's speech API
         try:
             self.request = urllib2.Request(url, audio, header)
         except: # try again before accepting defeat
-            logging.info("https request failed. %s, %s. trying again..." % (file, self.file))
-            time.sleep(0.2)
+            logging.info("https request failed. %s, %s. trying again..." % (filename, self.filename))
+            core.wait(0.2, hogCPUperiod=0)
             self.request = urllib2.Request(url, audio, header)
-    def _removeThread(self, gsqthread):
-        del core.runningThreads[core.runningThreads.index(gsqthread)]
     def getThread(self):
         """Send a query to google using a new thread, no blocking or timeout.
 
@@ -524,11 +720,8 @@ class Speech2Text(object):
         """
         gsqthread = _GSQueryThread(self.request)
         gsqthread.start()
-        core.runningThreads.append(gsqthread)
-        # this is the right idea, but need to .cancel() it when a response has come:
-        #threading.Timer(self.timeout, self._removeThread, (gsqthread,)).start()
         logging.info("Sending speech-recognition https request to google")
-        gsqthread.file = self.file
+        gsqthread.file = self.filename
         while not gsqthread.running:
             core.wait(0.001) # can return too quickly if thread is slow to start
         return gsqthread # word and time data will eventually be in the namespace
@@ -555,7 +748,7 @@ class Speech2Text(object):
         """
         gsqthread = self.getThread()
         while gsqthread.elapsed() < self.timeout:
-            time.sleep(0.1) # don't need precise timing to poll an http connection
+            core.wait(0.1, hogCPUperiod=0) # don't need precise timing to poll an http connection
             if not gsqthread.running:
                 break
         if gsqthread.running: # timed out
@@ -585,11 +778,12 @@ class BatchSpeech2Text(list):
             fileList = f
         else:
             fileList = list(files)
-        for i, file in enumerate(fileList):
-            gs = Speech2Text(file)
-            self.append( (file, gs.getThread()) ) # tuple
+        web.requireInternetAccess()  # needed to access google's speech API
+        for i, filename in enumerate(fileList):
+            gs = Speech2Text(filename)
+            self.append( (filename, gs.getThread()) ) # tuple
             if verbose:
-                print i, file
+                logging.info("%i %s" % (i, filename))
             while self._activeCount() >= maxThreads:
                 core.wait(.1, hogCPUPeriod=0) # idle at max count
     def _activeCount(self):
@@ -597,6 +791,21 @@ class BatchSpeech2Text(list):
         count = len([f for f,t in self if t.running and t.elapsed() <= self.timeout] )
         return count
 
+def _getFlacPath(flac_exe=''):
+    """Return full path to flac binary, using a cached value if possible.
+    """
+    global FLAC_PATH
+    if FLAC_PATH is None:
+        if sys.platform == 'win32':
+            if os.path.isfile(flac_exe):
+                FLAC_PATH = flac_exe
+            else:
+                FLAC_PATH = core.shellCall(['where', '/r', 'C:\\', 'flac'])
+        else:
+            FLAC_PATH = core.shellCall(['/usr/bin/which', 'flac'])
+        logging.info('set FLAC_PATH to %s' % FLAC_PATH)
+    return FLAC_PATH
+
 def switchOn(sampleRate=48000, outputDevice=None, bufferSize=None):
     """You need to switch on the microphone before use, which can take several seconds.
     The only time you can specify the sample rate (in Hz) is during switchOn().
@@ -641,49 +850,44 @@ def switchOn(sampleRate=48000, outputDevice=None, bufferSize=None):
     logging.exp('%s: switch on (%dhz) took %.3fs' % (__file__.strip('.py'), sampleRate, core.getTime() - t0))
 
 def switchOff():
-    """No longer needed; maintained for backwards compatibility only
+    """(No longer needed as of v1.76.00; retained for backwards compatibility.)
     """
-    return
+    logging.debug("microphone.switchOff() is deprecated; no longer needed.")
 
 if __name__ == '__main__':
     print ('\nMicrophone command-line testing\n')
     core.checkPygletDuringWait = False # don't dispatch events during a wait
     logging.console.setLevel(logging.DEBUG)
-    switchOn(16000) # import pyo, create a server
+    switchOn(48000) # import pyo, create a server
 
     mic = AudioCapture()
+    save = bool('--save' in sys.argv)
+    if save:
+        del sys.argv[sys.argv.index('--save')]
     if len(sys.argv) > 1: # stability test using argv[1] iterations
         for i in xrange(int(sys.argv[1])):
-            print i, mic.record(2)
-            mic.resample(8000, keep=False) # removes orig file
+            mic = AudioCapture()
+            print i, mic.record(1, block=True)
+            mic.resample(16000, keep=True)
             os.remove(mic.savedFile) # removes downsampled file
     else: # two interactive record + playback tests
-        testDuration = 2  # sec
         raw_input('testing record and playback, press <return> to start: ')
         print "say something:",
         sys.stdout.flush()
-        # tell it to record for 10s:
-        mic.record(testDuration * 5, block=False) # block False returns immediately
-            # which you want if you might need to stop a recording early
-        core.wait(testDuration)  # we'll stop the record after 2s, not 10
-        mic.stop()
-        print
-        print 'record stopped; sleeping 1s'
-        sys.stdout.flush()
-        core.wait(1)
-        print 'start playback ',
-        sys.stdout.flush()
-        mic.playback()
-        print 'end.', mic.savedFile
-        sys.stdout.flush()
-        os.remove(mic.savedFile)
-        mic.reset()
-
-        # do another record, fixed duration, use block=True
-        raw_input('<ret> for another: ')
-        print "say something else:",
-        sys.stdout.flush()
-        mic.record(testDuration, file='m') # block=True by default; here use explicit file name
-        mic.playback()
-        print mic.savedFile
-        os.remove(mic.savedFile)
+        try:
+            # tell it to record for 10s:
+            mic.record(10, block=False)  # returns immediately
+            core.wait(2)  # we'll stop the record after 2s
+            mic.stop()
+            print '\nrecord stopped; sleeping 1s'
+            sys.stdout.flush()
+            core.wait(1)
+            print 'start playback ',
+            sys.stdout.flush()
+            mic.playback()
+            print '\nend.', mic.savedFile
+        finally:
+            # delete the file even if Ctrl-C
+            if not save:
+                try: os.unlink(mic.savedFile)
+                except: pass
