@@ -61,7 +61,7 @@ for thisLibName in prefs.general['audioLib']:
             import pygame
             from pygame import mixer, sndarray
         else:
-            raise ValueError("Audio lib options are currently only 'pyo' or 'pygame', not '%'" %thisLibName)
+            raise ValueError("Audio lib options are currently only 'pyo' or 'pygame', not '%s'" %thisLibName)
     except:
         logging.warning('%s audio lib was requested but not loaded: %s' %(thisLibName, sys.exc_info()[1]))
         continue #to try next audio lib
@@ -350,7 +350,8 @@ class SoundPygame(_SoundBase):
 class SoundPyo(_SoundBase):
     """Create a sound object, from one of MANY ways.
     """
-    def __init__(self,value="C",secs=0.5,octave=4, stereo=True, sampleRate=44100, bits=16, name='', autoLog=True):
+    def __init__(self, value="C", secs=0.5, octave=4, stereo=True, volume=1.0,
+                 loop=False, sampleRate=44100, bits=16, name='', autoLog=True):
         """
         value: can be a number, string or an array.
 
@@ -374,7 +375,15 @@ class SoundPyo(_SoundBase):
             output sounds in the bottom octave (1) and the top
             octave (8) is generally painful
 
-        sampleRate(=44100): if the psychopy.sound.init() function has been called
+        stereo: True (= default, two channels left and right), False (one channel)
+
+        volume: loudness to play the sound, from 0.0 (silent) to 1.0 (max).
+            Adjustments are not possible during playback, only before.
+
+        loop: False (= default, just play once), or True (repeat indefinitely,
+            until `.stop()`)
+
+        sampleRate (= 44100): if the psychopy.sound.init() function has been called
             or if another sound has already been created then this argument will be
             ignored and the previous setting will be used
 
@@ -391,9 +400,12 @@ class SoundPyo(_SoundBase):
         self.autoLog=autoLog
         self.name=name
 
-        #try to create sound
+        #try to create sound; set volume and loop before setSound (else needsUpdate=True)
         self._snd=None
+        self.volume = min(1.0, max(0.0, volume))
+        self.loop = bool(loop)
         self.setSound(value=value, secs=secs, octave=octave)
+        self.needsUpdate = False
 
     def play(self, fromStart=True, log=True):
         """Starts playing the sound on an available channel.
@@ -401,10 +413,12 @@ class SoundPyo(_SoundBase):
 
         This runs off a separate thread i.e. your code won't wait for the
         sound to finish before continuing. You need to use a
-        psychopy.core.wait() command if you want things to pause.
-        If you call play() whiles something is already playing the sounds will
+        `psychopy.core.wait(mySound.getDuration())` if you want things to pause.
+        If you call `play()` while something is already playing the sounds will
         be played over each other.
         """
+        if self.needsUpdate:
+            self._updateSnd()  # ~0.00015s, regardless of the size of self._sndTable
         self._snd.out()
         self.status=STARTED
         if log and self.autoLog:
@@ -424,21 +438,36 @@ class SoundPyo(_SoundBase):
             logging.exp("Sound %s stopped" %(self.name), obj=self)
 
     def getDuration(self):
-        """Return the duration of the sound file
-        """
-        return self._sndTable.getDur()
-
+        """Return the duration of the sound"""
+        return self.duration
     def getVolume(self):
-        """Returns the current volume of the sound (0.0:1.0)"""
-        #ToDo : get volume for pyo
-        return volume
+        """Returns the current volume of the sound (0.0 to 1.0, inclusive)"""
+        return self.volume
+    def getLoop(self):
+        """Returns the current loop setting of the sound (True, False)"""
+        return self.loop
 
-    def setVolume(self,newVol, log=True):
-        """Sets the current volume of the sound (0.0:1.0)"""
-        #ToDo : set volume for pyo
-        pass
+    def setVolume(self, newVol, log=True):
+        """Sets the current volume of the sound (0.0 to 1.0, inclusive)"""
+        self.volume = min(1.0, max(0.0, newVol))
+        self.needsUpdate = True
+        if log and self.autoLog:
+            logging.exp("Sound %s set volume %.3f" % (self.name, self.volume), obj=self)
+        return self.getVolume()
+
+    def setLoop(self, newLoop, log=True):
+        """Sets the current loop (True or False"""
+        self.loop = (newLoop == True)
+        self.needsUpdate = True
+        if log and self.autoLog:
+            logging.exp("Sound %s set loop %s" % (self.name, self.loop), obj=self)
+        return self.getLoop()
+
+    def _updateSnd(self):
+        self.needsUpdate = False
+        self._snd = pyo.TableRead(self._sndTable, freq=self._sndTable.getRate(),
+                                  loop=self.loop, mul=self.volume)
     def _fromFile(self, fileName):
-
         #try finding the file
         self.fileName=None
         for filePath in ['', mediaLocation]:
@@ -450,17 +479,20 @@ class SoundPyo(_SoundBase):
             return False
         #load the file
         self._sndTable = pyo.SndTable(self.fileName)
-        self._snd = pyo.TableRead(self._sndTable, freq=self._sndTable.getRate(), loop=0)
+        self._updateSnd()
+        self.duration = self._sndTable.getDur()
         return True
 
     def _fromArray(self, thisArray):
-        #ToDo: create a pyo sound from an array
         if self.isStereo:
             channels=2
         else:
             channels=1
-        self._sndTable = pyo.DataTable(size=len(thisArray), init=thisArray.tolist(), chnls=channels)
-        self._snd = pyo.TableRead(self._sndTable, freq=self._sndTable.getRate(), loop=0)
+        self._sndTable = pyo.DataTable(size=len(thisArray), init=thisArray.tolist(),
+                                       chnls=channels)
+        self._updateSnd()
+        # a DataTable has no .getDur() method, so just store the duration:
+        self.duration = float(len(thisArray)) / self.sampleRate
         return True
 
 def initPygame(rate=22050, bits=16, stereo=True, buffer=1024):
@@ -501,18 +533,25 @@ def _bestDriver(devNames, devIDs):
             prefDriver = 'Primary Sound'
         #look for that driver in available devices
         for devN, devString in enumerate(devNames):
-            if prefDriver in devString:
-                audioDriver=devString
-                outputID=devIDs[devN]
-                return audioDriver, outputID #we found an asio driver don'w look for others
+            try:
+                if prefDriver.encode('utf-8') in devString.encode('utf-8'):
+                    audioDriver=devString
+                    outputID=devIDs[devN]
+                    return audioDriver, outputID #we found an asio driver don'w look for others
+            except UnicodeDecodeError, UnicodeEncodeError:
+                logging.warn('find best sound driver - could not interpret unicode in driver name')
+    else:
+        return None, None
 
 def initPyo(rate=44100, stereo=True, buffer=128):
     """setup the pyo (sound) server
     """
-    global pyoSndServer, Sound, audioDriver, duplex
+    global pyoSndServer, Sound, audioDriver, duplex, maxChnls
     Sound = SoundPyo
+    if not 'pyo' in locals():
+        import pyo  # microphone.switchOn() calls initPyo even if audioLib is something else
     #subclass the pyo.Server so that we can insert a __del__ function that shuts it down
-    class Server(pyo.Server):
+    class _Server(pyo.Server):
         core=core #make libs class variables so they don't get deleted first
         logging=logging
         def __del__(self):
@@ -521,51 +560,79 @@ def initPyo(rate=44100, stereo=True, buffer=128):
             self.shutdown()
             self.core.wait(0.5)#make sure enough time passes for the server to shutdown
             self.logging.debug('pyo sound server shutdown')#this may never get printed
+    if '.'.join(map(str, pyo.getVersion())) < '0.6.4':
+        Server = _Server
+    else:
+        Server = pyo.Server
 
-    #check if we already have a server and kill it
-    if globals().has_key('pyoSndServer') and hasattr(pyoSndServer,'shutdown'): #if it exists and isn't None!
-        #this doesn't appear to work!
+    # if we already have a server, just re-initialize it
+    if globals().has_key('pyoSndServer') and hasattr(pyoSndServer,'shutdown'):
         pyoSndServer.stop()
         core.wait(0.5)#make sure enough time passes for the server to shutdown
         pyoSndServer.shutdown()
-        pyoSndServer.reinit(sr=rate, nchnls=2, buffersize=buffer, duplex=1, audio=audioDriver)
+        core.wait(0.5)
+        pyoSndServer.reinit(sr=rate, nchnls=maxChnls, buffersize=buffer, audio=audioDriver)
         pyoSndServer.boot()
     else:
-        #create the instance of the server
         if platform=='win32':
             #check for output device/driver
             devNames, devIDs=pyo.pa_get_output_devices()
             audioDriver,outputID=_bestDriver(devNames, devIDs)
             if outputID:
                 logging.info('Using sound driver: %s (ID=%i)' %(audioDriver, outputID))
+                maxOutputChnls = pyo.pa_get_output_max_channels(outputID)
             else:
                 logging.warning('No audio outputs found (no speakers connected?')
                 return -1
             #check for valid input (mic)
             devNames, devIDs = pyo.pa_get_input_devices()
-            junk, inputID=_bestDriver(devNames, devIDs)
-            if inputID:
-                duplex=True
+            audioInputName, inputID = _bestDriver(devNames, devIDs)
+            if inputID is not None:
+                logging.info('Using sound-input driver: %s (ID=%i)' %(audioInputName, inputID))
+                maxInputChnls = pyo.pa_get_input_max_channels(inputID)
+                duplex = bool(maxInputChnls > 0)
             else:
                 duplex=False
-        else:#for other platforms set duplex to True
+        else:#for other platforms set duplex to True (if microphone is available)
             audioDriver = prefs.general['audioDriver'][0]
-            duplex=True
+            maxInputChnls = pyo.pa_get_input_max_channels(pyo.pa_get_default_input())
+            maxOutputChnls = pyo.pa_get_output_max_channels(pyo.pa_get_default_output())
+            duplex = bool(maxInputChnls > 0)
+
+        maxChnls = min(maxInputChnls, maxOutputChnls)
+        if maxInputChnls < 1:
+            logging.warning('%s.initPyo could not find microphone hardware; recording not available' % __name__)
+            maxChnls = maxOutputChnls
+        if maxOutputChnls < 1:
+            logging.error('%s.initPyo could not find speaker hardware; sound not available' % __name__)
+            return -1
+
+        # create the instance of the server:
         if platform=='darwin':
             #for mac we set the backend using the server audio param
-            pyoSndServer = Server(sr=rate, nchnls=2, buffersize=buffer, audio=audioDriver, duplex=duplex)
+            pyoSndServer = Server(sr=rate, nchnls=maxChnls, buffersize=buffer, audio=audioDriver)
         else:
             #with others we just use portaudio and then set the OutputDevice below
-            pyoSndServer = Server(sr=rate, nchnls=2, buffersize=buffer, duplex=duplex)
+            pyoSndServer = Server(sr=rate, nchnls=maxChnls, buffersize=buffer)
+
         pyoSndServer.setVerbosity(1)
         if platform=='win32':
             pyoSndServer.setOutputDevice(outputID)
             if inputID:
                 pyoSndServer.setInputDevice(inputID)
         #do other config here as needed (setDuplex? setOutputDevice?)
+        pyoSndServer.setDuplex(duplex)
         pyoSndServer.boot()
     core.wait(0.5)#wait for server to boot before starting te sound stream
     pyoSndServer.start()
+    try:
+        Sound()  # test creation, no play
+    except pyo.PyoServerStateException:
+        msg = "Failed to start pyo sound Server"
+        if platform == 'darwin' and audioDriver != 'portaudio':
+            msg += "; maybe try prefs.general.audioDriver 'portaudio'?"
+        logging.error(msg)
+        core.quit()
     logging.debug('pyo sound server started')
     logging.flush()
     
